/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.util;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import org.openide.util.Exceptions;

/**
 *
 * @author Gerben Feenstra
 */
public class TestUtils {
      
        /**
         * Return List of instantiated dummy arguments for the method.
         * 
         * Will return first enumeration (getEnumConstants()[0]) if the argument is an enum.
         * Will return simple numbers for primitive numbers.
         * Will return those same simple numbers for wrapped primitives
         * Will return a date of 100000 ms unless the name of the method started with "start" in which case it will return 0 ms (from epoch)
         * 
         * @param methodToInvoke
         * @return List of instantiated parameters for the method
         */
        public static List getDummyArgsFor(Method methodToInvoke) throws InstantiateArgumentException {
            // Create parameters for this method
            Class[] arguments = methodToInvoke.getParameterTypes();
            List varArgs = new ArrayList();
            for (Class argumentClass : arguments) {
                // newinstance does not work on enums
                if (argumentClass.isEnum()) {
                    varArgs.add(argumentClass.getEnumConstants()[0]);
                } else if (argumentClass.getName().equals("int")) {
                    varArgs.add(23);
                } else if (argumentClass.getName().equals("long")) {
                    varArgs.add(17L);
                } else if (argumentClass.getName().equals("double")) {
                    varArgs.add(13.0);
                } else if (argumentClass.getName().equals("char")) {
                    varArgs.add('c');
                } else if (argumentClass.getName().equals("float")) {
                    varArgs.add(7f);
                } else if (argumentClass.getName().equals("short")) {
                    varArgs.add(3);
                } else if (argumentClass.getName().equals("java.lang.Integer")) {
                    varArgs.add(new java.lang.Integer(23));
                } else if (argumentClass.getName().equals("java.lang.Long")) {
                    varArgs.add(new java.lang.Long(17L));
                } else if (argumentClass.getName().equals("java.lang.Double")) {
                    varArgs.add(new java.lang.Double(40.0));
                } else if (argumentClass.getName().equals("java.lang.Character")) {
                    varArgs.add(new java.lang.Character('c'));
                } else if (argumentClass.getName().equals("java.lang.Float")) {
                    varArgs.add(new java.lang.Float(13f));
                } else if (argumentClass.getName().equals("java.lang.Short")) {
                    varArgs.add(new java.lang.Short("3"));
                } // Startdates should be before enddates, so when methodname contains "start" use an earlier date then any other date for this object/class
                else if (argumentClass.getName().equals("java.util.Date")) {
                    if (methodToInvoke.getName().toLowerCase().contains("start")) {
                        varArgs.add(new java.util.Date(0));
                    } else {
                        varArgs.add(new java.util.Date(100000));
                    }

                } else if (argumentClass.getName().equals("java.awt.Color")) {
                    varArgs.add(java.awt.Color.BLUE);
                } // In any other case instantiate the class and add to the varargs array
                else {
                    try {
                        varArgs.add(argumentClass.newInstance());
                    } catch (InstantiationException ex) {
                        throw new InstantiateArgumentException("Cannot instantiate Argument " + argumentClass.getName() + " for method " + methodToInvoke);
                    } catch (IllegalAccessException ex) {
                        throw new InstantiateArgumentException("Cannot access Argument " + argumentClass.getName() + " for method " + methodToInvoke);
                    }
                }
            }
            return varArgs;
        }
        
        /**
         * Return all private non-static fields of specified class.
         * 
         * @param bean
         * @return A list of all private non-static fields of specified bean-class.
         */
        public static List<Field> getPrivateNonStaticFields(Class clazz) {
            List<Field> privateFields = new ArrayList<Field>();
            Field[] fields = clazz.getDeclaredFields();
            for (java.lang.reflect.Field field : fields) {
                if (Arrays.asList(field.getModifiers()).contains(Modifier.PRIVATE) && !Arrays.asList(field.getModifiers()).contains(Modifier.STATIC)) {
                    privateFields.add(field);
                }
            }
            return privateFields;
        }  
        
        /**
         * Check double references of the specified Field. Field must be of type Collection
         * If the Beanclass (A) contains a collection of objects (B) that individually contain a 
         * reference back to an object A the collection may not be used in equals and hashcode checks. 
         * For example: A ProductionUnit has a list of Operations, but an operation also has a reference to a ProductionUnit, which in turn has the list of Operations etc.. 
         * This will result in loop-calling a HashCode() method and may result in very-hard-to-solve bugs.
         * 
         * @param privateField Field holding a variable of type Collection
         * @param bean Bean this Field is declared in
         * @return True if Field (Collection) has references to objects/beans of a type which refers back to the bean this field is declared in.
         */
        public static boolean isDoubleReferenced(Field privateField, Class clazz) throws GetterNotFoundException {
            Class fieldType = privateField.getType();

            String fieldName = privateField.getName().toLowerCase();
            boolean doubleReference = false;

            // First, check if the type of the field is a Collection type
            if (Collection.class.isAssignableFrom(fieldType)) {

                // Must retreive getter to determine generic type
                Method getter = null;
                Method[] beanMethods = clazz.getDeclaredMethods();
                for (Method method : beanMethods) {
                    if (method.getName().toLowerCase().equals("get" + fieldName)) {
                        getter = method;
                    }
                }

                // Get generic type of getter
                if (getter == null) {
                    throw new GetterNotFoundException("No getter found for field [" + privateField.getName() + "] in bean " + clazz.getName());
                } else {
                    Class typeArgClass = null;
                    Type returnType = getter.getGenericReturnType();

                    // Get returntype argument (generic)
                    if (returnType instanceof ParameterizedType) {
                        ParameterizedType type = (ParameterizedType) returnType;
                        Type[] typeArguments = type.getActualTypeArguments();
                        for (Type typeArgument : typeArguments) {
                            typeArgClass = (Class) typeArgument;
                        }
                    }

                    // If the generic type of this field/getter is of type Bean, we should check of this Bean has a reference back.
                    if (clazz.isAssignableFrom(typeArgClass)) {
                        Object typeArgBeanInstance = null;
                        try {
                            typeArgBeanInstance = (Object) typeArgClass.newInstance();
                        } catch (InstantiationException ex) {
                            Exceptions.printStackTrace(ex);
                        } catch (IllegalAccessException ex) {
                            Exceptions.printStackTrace(ex);
                        }

                        // Get privates of typeargbean
                        List<Field> fields = TestUtils.getPrivateNonStaticFields(typeArgBeanInstance.getClass());


                        for (Field field : fields) {
                            if (field.getType().equals(clazz)) {
                                doubleReference = true;
                            }
                        }
                    }
                }
            }
            return doubleReference;
        }  
        
        /**
         * Find a substring inside one of the elements of the specified List<String>
         * 
         * Ignores Uppercase
         * 
         * @param list List of Strings
         * @param subStrToFind String to find
         * @return True of the substring exists somewhere in one of the elements in the list.
         */
        public static boolean findSubStringInList(List<String> list, String subStrToFind) {
            for (String line : list) {
                String str = line.trim().toLowerCase();
                if (str.contains(subStrToFind)) {
                    return true;
                }
            }
            return false;
        }
        
        /**
         * Analyze java source file, return content of the as String specified method in a List of Strings.
         * @param file
         * @param methodDeclaration
         * @return 
         */
        public static List<String> getMethodLinesAsList(File file, String methodDeclaration) {

            BufferedReader in = null;
            int accoladeCounter = 0;
            List<String> methodLines = new ArrayList<String>();
            try {
                in = new BufferedReader(new FileReader(file));
                String line = "";
                line = in.readLine();
                while (line != null) {

                    line = line.toLowerCase().trim();
                    if (line.contains(methodDeclaration.toLowerCase())) {
                        if (!line.trim().startsWith("//")) {
                            // Start di-secting the equals method

                            while (line != null) {

                                if (line.contains("{")) {
                                    accoladeCounter++;
                                }
                                if (line.contains("}")) {
                                    accoladeCounter--;
                                }
                                if (accoladeCounter > 0) {
                                    methodLines.add(line);
                                }
                                if (methodLines.size() > 0 && accoladeCounter == 0) {
                                    break;
                                }
                                line = in.readLine();
                            }
                        }
                    }
                    if (methodLines.size() > 0) {
                        break;
                    }

                    line = in.readLine();
                }
            } catch (FileNotFoundException ex) {
                Exceptions.printStackTrace(ex);
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            } finally {
                try {
                    in.close();
                } catch (IOException ex) {
                    Exceptions.printStackTrace(ex);
                }
            }
            return methodLines;
        }
        
        /**
         * Retrieve and return the path of the current module
         * Creates and returns the real (full) path as File object of the module + specified path inside the module
         * 
         * @param relPath For example: BeanTestUtils.getRealPath("/src/main/java");
         * @return For example: D:\projecten\agrosense~core\model\src\main\java
         */
        public static File getRealPath(String relPath){
            File currentpathFile = new File(".");
            String currentpathString = "";
            try {
                currentpathString = currentpathFile.getCanonicalPath();

            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }
            
            String[] pathMembers = relPath.split("/");            
            
            StringBuilder sb = new StringBuilder();
            sb.append(currentpathString);
            
            for(String pathMember : pathMembers){
                sb.append(File.separator);
                sb.append(pathMember);
            }
            // TODO: Check is path is valid?
            
            return new File(sb.toString());
        }
        
        /**
         * Creates a list of keys in the specified (crop) bundle files
         * @param bundleFile The bundlefile to find keys in
         * @return List of keys
         */
        public static List<String> getAllKeysFrom(File bundleFile){
            List<String> keys = new ArrayList<String>();
            try {
                FileInputStream fileInputStream = new FileInputStream(bundleFile);
                DataInputStream dataInputStream = new DataInputStream(fileInputStream);
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(dataInputStream));

                String strLine;
                //Read File Line By Line
                while ((strLine = bufferedReader.readLine()) != null) {
                    strLine = strLine.trim();
                    if (!strLine.equals("") && !strLine.startsWith("#") && strLine.contains("=")) {
                        String[] line = strLine.split("=");
                        keys.add(line[0]);
                    }
                }
                //Close the input stream
                fileInputStream.close();
            } catch (Exception e) {
                System.err.println("Error: " + e.getMessage());
            }
            return keys;
        }
}
